

{- |
Copyright  : Will Thompson, Iñaki García Etxebarria and Jonas Platte
License    : LGPL-2.1
Maintainer : Iñaki García Etxebarria (garetxe@gmail.com)

A GOptionEntry struct defines a single option. To have an effect, they
must be added to a #GOptionGroup with g_option_context_add_main_entries()
or g_option_group_add_entries().
-}

module GI.GLib.Structs.OptionEntry
    ( 

-- * Exported types
    OptionEntry(..)                         ,
    newZeroOptionEntry                      ,
    noOptionEntry                           ,


 -- * Properties
-- ** Arg
    optionEntryReadArg                      ,


-- ** ArgData
    optionEntryReadArgData                  ,


-- ** ArgDescription
    optionEntryReadArgDescription           ,


-- ** Description
    optionEntryReadDescription              ,


-- ** Flags
    optionEntryReadFlags                    ,


-- ** LongName
    optionEntryReadLongName                 ,


-- ** ShortName
    optionEntryReadShortName                ,




    ) where

import Prelude ()
import Data.GI.Base.ShortPrelude

import qualified Data.Text as T
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map

import GI.GLib.Types
import GI.GLib.Callbacks

newtype OptionEntry = OptionEntry (ForeignPtr OptionEntry)
-- | Construct a `OptionEntry` struct initialized to zero.
newZeroOptionEntry :: MonadIO m => m OptionEntry
newZeroOptionEntry = liftIO $ callocBytes 48 >>= wrapPtr OptionEntry

noOptionEntry :: Maybe OptionEntry
noOptionEntry = Nothing

optionEntryReadLongName :: OptionEntry -> IO T.Text
optionEntryReadLongName s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 0) :: IO CString
    val' <- cstringToText val
    return val'

optionEntryReadShortName :: OptionEntry -> IO Int8
optionEntryReadShortName s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 8) :: IO Int8
    return val

optionEntryReadFlags :: OptionEntry -> IO Int32
optionEntryReadFlags s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 12) :: IO Int32
    return val

optionEntryReadArg :: OptionEntry -> IO OptionArg
optionEntryReadArg s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 16) :: IO CUInt
    let val' = (toEnum . fromIntegral) val
    return val'

optionEntryReadArgData :: OptionEntry -> IO (Ptr ())
optionEntryReadArgData s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 24) :: IO (Ptr ())
    return val

optionEntryReadDescription :: OptionEntry -> IO T.Text
optionEntryReadDescription s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 32) :: IO CString
    val' <- cstringToText val
    return val'

optionEntryReadArgDescription :: OptionEntry -> IO T.Text
optionEntryReadArgDescription s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 40) :: IO CString
    val' <- cstringToText val
    return val'

type family ResolveOptionEntryMethod (t :: Symbol) (o :: *) :: * where
    ResolveOptionEntryMethod l o = MethodResolutionFailed l o

instance (info ~ ResolveOptionEntryMethod t OptionEntry, MethodInfo info OptionEntry p) => IsLabelProxy t (OptionEntry -> p) where
    fromLabelProxy _ = overloadedMethod (MethodProxy :: MethodProxy info)

#if MIN_VERSION_base(4,9,0)
instance (info ~ ResolveOptionEntryMethod t OptionEntry, MethodInfo info OptionEntry p) => IsLabel t (OptionEntry -> p) where
    fromLabel _ = overloadedMethod (MethodProxy :: MethodProxy info)
#endif


